
#include "FreeRTOS.h"
#include "task.h"
#include "main.h"
#include "cmsis_os.h"

#include "tcpclient.h"
#include "wizInterface.h"

#include <stdio.h>

#define SERVER_IP1	192	//server ip
#define SERVER_IP2	168
#define SERVER_IP3	1
#define SERVER_IP4	227
#define SERVER_PORT	1234 //server port

static uint8_t serverIP[] = { SERVER_IP1, SERVER_IP2, SERVER_IP3, SERVER_IP4 };
static uint16_t localPort = 5000;

//tcp client task
void StartWizTcpClientTask(void const *argument) {

	// WIZ_ChipInit ȣϿ ʱȭ
	if(!WIZ_ChipInit())
	{
		printf("WIZ_ChipInit failed.\n");
		vTaskDelete(NULL);	//clear task
	}

	// WIZ_NetworkInit ȣϿ Ʈũ  (IP  Ǵ DHCP   IP Ҵ)
	if(!WIZ_NetworkInit())
	{
		printf("WIZ_NetworkInit failed.\n");
		vTaskDelete(NULL);	//clear task
	}

	//printf("Ver:%x\n", getVERSIONR());
	//printf("TTL:%x\n", getSn_TTL(2));
	//printf("RTR:%d\n", getRTR());


	while (1) 
	{
		int32_t ret;

		// socket API  ȣϿ  
		ret = socket(CLIENT_SOCKET, Sn_MR_TCP, localPort++, SF_TCP_NODELAY);

		if(localPort == 0xFFFF) {
			localPort = 5000;
		}

		if (ret < 0) {
			//HAL_GPIO_TogglePin(LED_RED_GPIO_Port, LED_RED_Pin);
			printf("socket failed{%ld}.\n", ret);
			close(CLIENT_SOCKET);
			osDelay(500);
			continue;
		}

		while(getSn_SR(CLIENT_SOCKET) != SOCK_INIT);	//  µɶ 

		printf("socket open.\n");

		//connect API  ȣϿ  
		//modified connect api to make a detour invalid error returns(SOCKERR_SOCKCLOSED)
		ret = connect(CLIENT_SOCKET, serverIP, SERVER_PORT);
		if (ret < 0) {
			//HAL_GPIO_TogglePin(LED_RED_GPIO_Port, LED_RED_Pin);
			printf("connect failed{%ld}.Sn_SR()={%d}\n", ret, getSn_SR(CLIENT_SOCKET));
			//modified connect api to make a detour infinite loop when Sn_SR returns 0x10
			close(CLIENT_SOCKET);
			osDelay(500);
			continue;
		}

		printf("connect!!\n");

		//prepare data to send & receive
		struct time_packet packet;
		memset(&packet, 0, sizeof(struct time_packet));
		packet.head = 0xAE;	//head
		packet.type = REQ; 	//request type
		packet.tail = 0xEA; //tail

		uint8_t failed = 0;
		int16_t written = 0;
		int16_t read = 0;

		//send API  ȣϿ REQ 
		ret = send(CLIENT_SOCKET, (uint8_t*) (&packet + written), sizeof(struct time_packet) - written);
		if (ret < 0) {
			//HAL_GPIO_TogglePin(LED_RED_GPIO_Port, LED_RED_Pin);
			printf("send failed{%ld}.\n", ret);
			close(CLIENT_SOCKET); //unexpected close
			continue;
		}

		printf("send~\n");

		//recv API  ȣϿ RESP 
		while (1) {
			ret = recv(CLIENT_SOCKET, (uint8_t*) (&packet + read), sizeof(struct time_packet) - read);
			if (ret < 0) {
				//HAL_GPIO_TogglePin(LED_RED_GPIO_Port, LED_RED_Pin);
				printf("recv failed.{%ld}\n", ret);
				close(CLIENT_SOCKET); //unexpected close
				failed = 1;
				break;
			}

			read += ret;
			if (read >= sizeof(struct time_packet))	//overflow
				break;
		}

		if (failed)
			continue; //start again

		 //if received length is valid, print time information & toggle led
		if (read == sizeof(struct time_packet) && packet.type == RESP)
				{
			printf("%04d-%02d-%02d %02d:%02d:%02d\n",
					packet.year + 2000, packet.month, packet.day,
					packet.hour, packet.minute, packet.second);
			//HAL_GPIO_TogglePin(LED_BLUE_GPIO_Port, LED_BLUE_Pin);
		}
		close(CLIENT_SOCKET);		//close API  ȣϿ  close
		osDelay(10);				//10ms delay     ݺ
	}

	vTaskDelete(NULL);	//clear task
}
